`timescale 1ns/1ps

// Testbench 代码
module testbench();

  // 时钟信号 复位信号
  reg clk, rst_n, start;
  always #5 clk = !clk;

  // 生成输入信号
  reg [2:0] A[0:3];
  reg [2:0] B[0:3];
  // 模拟Ram的port
  reg [2:0] Ram_A_Data;
  wire [1:0] Ram_A_Addr;
  wire Ram_A_Enable;
  reg [2:0] Ram_B_Data;
  wire [1:0] Ram_B_Addr;
  wire Ram_B_Enable;
  // 模拟输出Ram的port
  wire [5:0] Ram_Result_Data;
  wire [1:0] Ram_Result_Addr;
  wire Ram_Result_Enable;
  // 接收输出信号
  reg [5:0] C[0:3];
  reg [5:0] Result[0:3];

  // Ram A 模拟Ram读功能  
  always@(posedge clk, negedge rst_n) begin
    if (rst_n == 1'b0) begin
      Ram_A_Data <= 3'b0;
    end
    else if(Ram_A_Enable) begin
      Ram_A_Data <= A[Ram_A_Addr];
    end
  end
  // Ram B 模拟Ram读功能  
  always@(posedge clk, negedge rst_n) begin
    if (rst_n == 1'b0) begin
      Ram_B_Data <= 3'b0;
    end
    else if(Ram_B_Enable) begin
      Ram_B_Data <= B[Ram_B_Addr];
    end
  end
  integer k;
  //Ram Result 模拟Ram写功能
  always@(posedge clk, negedge rst_n) begin
    if (rst_n == 1'b0) begin
      for (k=0; k<4; k=k+1) begin
        Result[k] <= 6'b0;
      end
    end
    else if(Ram_Result_Enable) begin
      Result[Ram_Result_Addr] <= Ram_Result_Data;
    end
  end
  
  integer n;
  integer i,j,u,v;
  // 调用任务进行计算，并将计算结果输出到 C 矩阵中
  initial begin
    // 定义输入矩阵 A 和 B
    $readmemh("./data/A.txt", A);
    $readmemh("./data/B.txt", B);

    for (i=0; i<4; i=i+1) begin
      C[i] = 0;
      u = i/2; 
      v = i%2;
      for (j=0; j<2; j=j+1) begin
        C[i] = C[i] + A[u*2+j] * B[j*2+v];
      end
    end   
    // 输出计算结果
    $display("~~~~~~~~~~~~~~~~~~~ Right_Result ~~~~~~~~~~~~~~~~~~~");
    for (n=0; n<2; n=n+1) begin
      $display("                      %d, %d                     ",C[n*2+0], C[n*2+1]);
    end
    
    clk = 1'b1;
    rst_n = 1'b0;
    start = 1'b0;

    # 20;
    rst_n = 1'b1;
    # 20;
    start = 1'b1;
    # 960; // 1000ns时启动比较函数，请在此之前完成矩阵乘法，并存入Result中
    $display("~~~~~~~~~~~~~~~~~~~ Your_Result ~~~~~~~~~~~~~~~~~~~");
    for (n=0; n<2; n=n+1) begin
      $display("                      %d, %d                     ",Result[n*2+0], Result[n*2+1]);
    end
    compare();
    writePredict();
    $stop;
  end


  // 例化matrix_multi模块
  matrix_multi inst(
    .clk(clk),
    .rst_n(rst_n),
    .start(start),
    .Ram_A_Data(Ram_A_Data),
    .Ram_A_Addr(Ram_A_Addr),
    .Ram_B_Data(Ram_B_Data),
    .Ram_B_Addr(Ram_B_Addr),
    .Ram_Result_Data(Ram_Result_Data),
    .Ram_Result_Addr(Ram_Result_Addr),
    .Ram_A_Enable(Ram_A_Enable),
    .Ram_B_Enable(Ram_B_Enable),
    .Ram_Result_Enable(Ram_Result_Enable)
  );

task writePredict;
integer i,file;
begin
    file = $fopen("result.txt", "w+");
    $fwrite(file,"~~~~~~~~~~~~~~~~~~~ Right_Result ~~~~~~~~~~~~~~~~~~~\n");
    for(i = 0;i < 2;i = i + 1) begin
        $fwrite(file,"                      %d, %d                     \n",C[i*2+0], C[i*2+1]);
    end
    $fwrite(file,"~~~~~~~~~~~~~~~~~~~ Your_Result ~~~~~~~~~~~~~~~~~~~\n");
    for(i = 0;i < 2;i = i + 1) begin
        $fwrite(file,"                      %d, %d                     \n",Result[i*2+0], Result[i*2+1]);
    end
    $fclose(file);
end
endtask

task compare;
integer  i;
integer  j;
begin
    j = 0;
    for(i = 0;i < 4 ;i = i + 1) begin
        if(C[i] != Result[i]) begin
            j = j + 1;
        end
    end
    if(j == 0) begin
        $display("~~~~~~~~~~~~~~~~~~~ TEST_PASS ~~~~~~~~~~~~~~~~~~~");
        $display("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
        $display("~~~~~~~~~ #####     ##     ####    #### ~~~~~~~~~");
        $display("~~~~~~~~~ #    #   #  #   #       #     ~~~~~~~~~");
        $display("~~~~~~~~~ #    #  #    #   ####    #### ~~~~~~~~~");
        $display("~~~~~~~~~ #####   ######       #       #~~~~~~~~~");
        $display("~~~~~~~~~ #       #    #  #    #  #    #~~~~~~~~~");
        $display("~~~~~~~~~ #       #    #   ####    #### ~~~~~~~~~");
        $display("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
    end
    else begin
        $display("~~~~~~~~~~~~~~~~~~~ TEST_FAIL ~~~~~~~~~~~~~~~~~~~~");
        $display("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
        $display("~~~~~~~~~~######    ##       #    #     ~~~~~~~~~~");
        $display("~~~~~~~~~~#        #  #      #    #     ~~~~~~~~~~");
        $display("~~~~~~~~~~#####   #    #     #    #     ~~~~~~~~~~");     
        $display("~~~~~~~~~~#       ######     #    #     ~~~~~~~~~~");
        $display("~~~~~~~~~~#       #    #     #    #     ~~~~~~~~~~");
        $display("~~~~~~~~~~#       #    #     #    ######~~~~~~~~~~");
        $display("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
    end
end
endtask 
endmodule
